//////////////////////////////////////////////////////////////////////////////
//
// Copyright: (c)1997-2004 ALTITECH
// Filename:  pciclass.h (PCI-V3.0)
// Overview:  This include file contains PCI class code definitions
// Author:    William H. Avery III
//
// Contents:
//   This header file includes a set of custom "Class Code" strings for PCI
//   development, prototyping & generic information.  This is useful in
//   determining the general nature of a particular PCI device.  use
//   sizeof(<table>) / sizeof(<structure>) to determine total table entries.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef __PCICLASS_INCLUDE__ // Save Time, Ensure One Time Pass Only
#define __PCICLASS_INCLUDE__

//////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////
//
// String constants (saves space)
//
////////////////////////////////////////

static char NM_NULL[]       = "";
#define NM_N (char*)NM_NULL
static char NM_OTHER[]      = "Other";
static char NM_GENERIC[]    = "Generic";
static char NM_UNKNOWN[]    = "Unknown";

// PCI Base Names
static char NM_BASE0[]      = "Pre-2.0 PCI Device";
static char NM_BASE1[]      = "Mass Storage Controller";
static char NM_BASE2[]      = "Network Controller";
static char NM_BASE3[]      = "Display Controller";
static char NM_BASE4[]      = "Multimedia Device";
static char NM_BASE5[]      = "Memory Controller";
static char NM_BASE6[]      = "Bridge Device";
static char NM_BASE7[]      = "Simple Comm Controller";
static char NM_BASE8[]      = "Base Systems Peripheral";
static char NM_BASE9[]      = "Input Device";
static char NM_BASE10[]     = "Docking Station";
static char NM_BASE11[]     = "Processor";
static char NM_BASE12[]     = "Serial Bus Controller";
static char NM_BASE13[]     = "Wireless Controller";
static char NM_BASE14[]     = "Intelligent IO Controller";
static char NM_BASE15[]     = "Satellite Comm Controller";
static char NM_BASE16[]     = "Encrypt/Decrypt Controller";
static char NM_BASE17[]     = "Data Acq/Signal Processing";

// Other Duplicate Table Names

static char NM_PCCOMPAT[]   = "PC Compatible";
static char NM_RACEWAY[]    = "PCI->RACEway";
static char NM_PCIPCISEMI[] = "PCI->PCI Semi-Transparent";

static char NM_SERIAL[]     = "Serial";
static char NM_PARALLEL[]   = "Parallel";
static char NM_MODEM[]      = "Modem";
static char NM_16450[]      = "16450 Compatible";
static char NM_16550[]      = "16550 Compatible";
static char NM_16650[]      = "16650 Compatible";
static char NM_16750[]      = "16750 Compatible";
static char NM_16850[]      = "16850 Compatible";
static char NM_16950[]      = "16950 Compatible";

static char NM_GEN8259[]    = "Generic 8259";
static char NM_PIC[]        = "PIC (Pgm Int Controller)";
static char NM_DMA[]        = "DMA (Direct Mem Access)";
static char NM_SYSTIMER[]   = "System Timer";
static char NM_RTC[]        = "RTC (Real Time Clock)";

static char NM_GAMEPORT[]   = "Gameport";

static char NM_FIREWIRE[]   = "Firewire (IEEE 1394)";
static char NM_USB[]        = "USB (Universal Serial Bus)";
static char NM_IPMI[]       = "IPMI (Intelligent Per Mgmt)";

static char NM_ETHERNET[]   = "Ethernet";

////////////////////////////////////////
//
// Master Class Code Table
//
////////////////////////////////////////

typedef struct _PCI_CLASSCODETABLE
{
  unsigned char	BaseClass;
  unsigned char	SubClass;
  unsigned char	ProgIf;
  char *BaseDesc;
  char *SubDesc;
  char *ProgDesc;
} PCI_CLASSCODETABLE, *PPCI_CLASSCODETABLE;

static PCI_CLASSCODETABLE PciClassCodeTable[]=
{
  // 0x00 - Pre-2.0 PCI Specification Device
  { 0x00, 0x00, 0x00, (char*)NM_BASE0,  "Non-VGA",          NM_N },
  { 0x00, 0x01, 0x00, (char*)NM_BASE0,  "VGA Compatible",   NM_N },
  { 0x00, 0x80, 0x00, (char*)NM_BASE0,  (char*)NM_OTHER,    NM_N }, // Not PCI

  // 0x01 - Mass Storage Controller
  { 0x01, 0x00, 0x00, (char*)NM_BASE1,  "SCSI",             NM_N },
  { 0x01, 0x01, 0x00, (char*)NM_BASE1,  "IDE",              NM_N },
  { 0x01, 0x01, 0x80, (char*)NM_BASE1,  "IDE",              "Master" },
  { 0x01, 0x02, 0x00, (char*)NM_BASE1,  "Floppy",           NM_N },
  { 0x01, 0x03, 0x00, (char*)NM_BASE1,  "IPI",              NM_N },
  { 0x01, 0x04, 0x00, (char*)NM_BASE1,  "RAID",             NM_N },
  { 0x01, 0x05, 0x00, (char*)NM_BASE1,  "ATA",              NM_N },             // PCI 2.3
  { 0x01, 0x05, 0x20, (char*)NM_BASE1,  "ATA",              "ADMA - Single" },  // PCI 2.3
  { 0x01, 0x05, 0x30, (char*)NM_BASE1,  "ATA",              "ADMA - Chained" }, // PCI 2.3
  { 0x01, 0x06, 0x00, (char*)NM_BASE1,  "Serial ATA",       (char*)NM_GENERIC },// PCI 3.0
  { 0x01, 0x06, 0x01, (char*)NM_BASE1,  "Serial ATA",       "AHCI 1.0" },       // PCI 3.0
  { 0x01, 0x07, 0x00, (char*)NM_BASE1,  "SAS (Serial SCSI)",NM_N },             // PCI 3.0
  { 0x01, 0x80, 0x00, (char*)NM_BASE1,  (char*)NM_OTHER,    NM_N },

  // 0x02 - Network Controller
  { 0x02, 0x00, 0x00, (char*)NM_BASE2,  "Ethernet",         NM_N},
  { 0x02, 0x01, 0x00, (char*)NM_BASE2,  "Token Ring",       NM_N },
  { 0x02, 0x02, 0x00, (char*)NM_BASE2,  "FDDI",             NM_N },
  { 0x02, 0x03, 0x00, (char*)NM_BASE2,  "ATM",              NM_N },
  { 0x02, 0x04, 0x00, (char*)NM_BASE2,  "ISDN",             NM_N },
  { 0x02, 0x05, 0x00, (char*)NM_BASE2,  "WorldFip",         NM_N }, // PCI 2.3
  { 0x02, 0x06, 0x00, (char*)NM_BASE2,  "PICMG 2.14 Multi", NM_N }, // PCI 2.3
  { 0x02, 0x80, 0x00, (char*)NM_BASE2,  (char*)NM_OTHER,    NM_N },

  // 0x03 - Display Controller
  { 0x03, 0x00, 0x00, (char*)NM_BASE3,  (char*)NM_PCCOMPAT, "VGA" },
  { 0x03, 0x00, 0x01, (char*)NM_BASE3,  (char*)NM_PCCOMPAT, "8514" },
  { 0x03, 0x01, 0x00, (char*)NM_BASE3,  "XGA",              NM_N },
  { 0x03, 0x02, 0x00, (char*)NM_BASE3,  "3D",               NM_N },
  { 0x03, 0x80, 0x00, (char*)NM_BASE3,  (char*)NM_OTHER,    NM_N },

  // 0x04 - Multimedia Device
  { 0x04, 0x00, 0x00, (char*)NM_BASE4,  "Video",            NM_N },
  { 0x04, 0x01, 0x00, (char*)NM_BASE4,  "Audio",            NM_N },
  { 0x04, 0x02, 0x00, (char*)NM_BASE4,  "Telephony",        NM_N },
  { 0x04, 0x80, 0x00, (char*)NM_BASE4,  (char*)NM_OTHER,    NM_N },

  // 0x05 - Memory Controller
  { 0x05, 0x00, 0x00, (char*)NM_BASE5,  "RAM",              NM_N },
  { 0x05, 0x01, 0x00, (char*)NM_BASE5,  "Flash",            NM_N },
  { 0x05, 0x80, 0x00, (char*)NM_BASE5,  (char*)NM_OTHER,    NM_N },

  // 0x06 - Bridge Device
  { 0x06, 0x00, 0x00, (char*)NM_BASE6,  "PCI->Host",        NM_N },
  { 0x06, 0x01, 0x00, (char*)NM_BASE6,  "PCI->ISA",         NM_N },
  { 0x06, 0x02, 0x00, (char*)NM_BASE6,  "PCI->EISA",        NM_N },
  { 0x06, 0x03, 0x00, (char*)NM_BASE6,  "PCI->MCA",         NM_N },
  { 0x06, 0x04, 0x00, (char*)NM_BASE6,  "PCI->PCI",         NM_N },
  { 0x06, 0x04, 0x01, (char*)NM_BASE6,  "PCI->PCI",         "Subtractive Decode" },
  { 0x06, 0x05, 0x00, (char*)NM_BASE6,  "PCI->PCMCIA",      NM_N },
  { 0x06, 0x06, 0x00, (char*)NM_BASE6,  "PCI->NuBus",       NM_N },
  { 0x06, 0x07, 0x00, (char*)NM_BASE6,  "PCI->CardBus",     NM_N },
  { 0x06, 0x08, 0x00, (char*)NM_BASE6,  (char*)NM_RACEWAY,  "Transparent" },
  { 0x06, 0x08, 0x01, (char*)NM_BASE6,  (char*)NM_RACEWAY,  "End-Point" },  // PCI 3.0
  { 0x06, 0x09, 0x40, (char*)NM_BASE6,  (char*)NM_PCIPCISEMI, "Primary" },  // PCI 2.2
  { 0x06, 0x09, 0x80, (char*)NM_BASE6,  (char*)NM_PCIPCISEMI, "Secondary" },// PCI 2.2
  { 0x06, 0x0A, 0x00, (char*)NM_BASE6,  "InfiniBand->PCI",  NM_N }, // PCI 2.3
  { 0x06, 0x80, 0x00, (char*)NM_BASE6,  (char*)NM_OTHER,    NM_N },

  // 0x07 - Simple Communications Controller
  { 0x07, 0x00, 0x00, (char*)NM_BASE7,  (char*)NM_SERIAL,   (char*)NM_GENERIC },
  { 0x07, 0x00, 0x01, (char*)NM_BASE7,  (char*)NM_SERIAL,   (char*)NM_16450 },
  { 0x07, 0x00, 0x02, (char*)NM_BASE7,  (char*)NM_SERIAL,   (char*)NM_16550 },
  { 0x07, 0x00, 0x03, (char*)NM_BASE7,  (char*)NM_SERIAL,   (char*)NM_16650 },
  { 0x07, 0x00, 0x04, (char*)NM_BASE7,  (char*)NM_SERIAL,   (char*)NM_16750 },
  { 0x07, 0x00, 0x05, (char*)NM_BASE7,  (char*)NM_SERIAL,   (char*)NM_16850 },
  { 0x07, 0x00, 0x06, (char*)NM_BASE7,  (char*)NM_SERIAL,   (char*)NM_16950 },
  { 0x07, 0x01, 0x00, (char*)NM_BASE7,  (char*)NM_PARALLEL, "Standard" },
  { 0x07, 0x01, 0x01, (char*)NM_BASE7,  (char*)NM_PARALLEL, "Bidirectional" },
  { 0x07, 0x01, 0x02, (char*)NM_BASE7,  (char*)NM_PARALLEL, "ECP V1.X" },
  { 0x07, 0x01, 0x03, (char*)NM_BASE7,  (char*)NM_PARALLEL, "IEEE 1284 Master" },
  { 0x07, 0x01, 0xFE, (char*)NM_BASE7,  (char*)NM_PARALLEL, "IEEE 1284 Target" },
  { 0x07, 0x02, 0x00, (char*)NM_BASE7,  "Multiport",         NM_N },
  { 0x07, 0x03, 0x00, (char*)NM_BASE7,  (char*)NM_MODEM,    (char*)NM_GENERIC },
  { 0x07, 0x03, 0x01, (char*)NM_BASE7,  (char*)NM_MODEM,    (char*)NM_16450 },
  { 0x07, 0x03, 0x02, (char*)NM_BASE7,  (char*)NM_MODEM,    (char*)NM_16550 },
  { 0x07, 0x03, 0x03, (char*)NM_BASE7,  (char*)NM_MODEM,    (char*)NM_16650 },
  { 0x07, 0x03, 0x04, (char*)NM_BASE7,  (char*)NM_MODEM,    (char*)NM_16750 },
  { 0x07, 0x04, 0x00, (char*)NM_BASE7,  "GPIB (IEEE 488.1/2)", NM_N },// PCI 2.3
  { 0x07, 0x05, 0x00, (char*)NM_BASE7,  "Smart Card",        NM_N },  // PCI 2.3
  { 0x07, 0x80, 0x02, (char*)NM_BASE7,  (char*)NM_OTHER,     NM_N },

  // 0x08 - Base Systems Peripheral
  // PIC = Programmable Interrupt Controller
  { 0x08, 0x00, 0x00, (char*)NM_BASE8,  (char*)NM_PIC,       (char*)NM_GEN8259 },
  { 0x08, 0x00, 0x01, (char*)NM_BASE8,  (char*)NM_PIC,       "ISA" },
  { 0x08, 0x00, 0x02, (char*)NM_BASE8,  (char*)NM_PIC,       "PCI" },
  { 0x08, 0x00, 0x10, (char*)NM_BASE8,  (char*)NM_PIC,       "I/O APIC" },
  { 0x08, 0x00, 0x20, (char*)NM_BASE8,  (char*)NM_PIC,       "I/O(x) APIC" },
  { 0x08, 0x01, 0x00, (char*)NM_BASE8,  (char*)NM_DMA,       (char*)NM_GEN8259 },
  { 0x08, 0x01, 0x01, (char*)NM_BASE8,  (char*)NM_DMA,       "ISA" },
  { 0x08, 0x01, 0x02, (char*)NM_BASE8,  (char*)NM_DMA,       "EISA" },
  { 0x08, 0x02, 0x00, (char*)NM_BASE8,  (char*)NM_SYSTIMER,  (char*)NM_GEN8259 },
  { 0x08, 0x02, 0x01, (char*)NM_BASE8,  (char*)NM_SYSTIMER,  "ISA" },
  { 0x08, 0x02, 0x02, (char*)NM_BASE8,  (char*)NM_SYSTIMER,  "EISA" },
  { 0x08, 0x03, 0x00, (char*)NM_BASE8,  (char*)NM_RTC,       (char*)NM_GENERIC },
  { 0x08, 0x03, 0x01, (char*)NM_BASE8,  (char*)NM_RTC,       "ISA" },
  { 0x08, 0x04, 0x00, (char*)NM_BASE8,  "PCI Hot-Plug",      (char*)NM_GENERIC },
  { 0x08, 0x80, 0x00, (char*)NM_BASE8,  (char*)NM_OTHER,     NM_N },

   // 0x09 - Input Device
  { 0x09, 0x00, 0x00, (char*)NM_BASE9,  "Keyboard",          NM_N },
  { 0x09, 0x01, 0x00, (char*)NM_BASE9,  "Digitizer (Pen)",   NM_N },
  { 0x09, 0x02, 0x00, (char*)NM_BASE9,  "Mouse",             NM_N },
  { 0x09, 0x03, 0x00, (char*)NM_BASE9,  "Scanner",           NM_N },
  { 0x09, 0x04, 0x00, (char*)NM_BASE9,  (char*)NM_GAMEPORT,  (char*)NM_GENERIC },
  { 0x09, 0x04, 0x10, (char*)NM_BASE9,  (char*)NM_GAMEPORT,  "Legacy" },
  { 0x09, 0x80, 0x00, (char*)NM_BASE9,  (char*)NM_OTHER,     NM_N },

  // 0x0A - Docking Station
  { 0x0A, 0x00, 0x00, (char*)NM_BASE10, (char*)NM_GENERIC,   NM_N },
  { 0x0A, 0x80, 0x00, (char*)NM_BASE10, (char*)NM_OTHER,     NM_N },

  // 0x0B - Processor
  { 0x0B, 0x00, 0x00, (char*)NM_BASE11, "i386",              NM_N },
  { 0x0B, 0x01, 0x00, (char*)NM_BASE11, "i486",              NM_N },
  { 0x0B, 0x02, 0x00, (char*)NM_BASE11, "Pentium",           NM_N },
  { 0x0B, 0x03, 0x00, (char*)NM_BASE11, "Pentium Pro",       NM_N },
  { 0x0B, 0x10, 0x00, (char*)NM_BASE11, "Alpha",             NM_N },
  { 0x0B, 0x20, 0x00, (char*)NM_BASE11, "Power PC",          NM_N },
  { 0x0B, 0x30, 0x00, (char*)NM_BASE11, "MIPS",              NM_N },
  { 0x0B, 0x40, 0x00, (char*)NM_BASE11, "Co-processor",      NM_N },
  { 0x0B, 0x80, 0x00, (char*)NM_BASE11, (char*)NM_OTHER,     NM_N }, // Not PCI

  // 0x0C - Serial Bus Controller
  { 0x0C, 0x00, 0x00, (char*)NM_BASE12, (char*)NM_FIREWIRE,  (char*)NM_GENERIC },
  { 0x0C, 0x00, 0x10, (char*)NM_BASE12, (char*)NM_FIREWIRE,  "OHCI" },
  { 0x0C, 0x01, 0x00, (char*)NM_BASE12, "ACCESS.bus",        NM_N },
  { 0x0C, 0x02, 0x00, (char*)NM_BASE12, "SSA",               NM_N },
  { 0x0C, 0x03, 0x00, (char*)NM_BASE12, (char*)NM_USB,       "UHCI" },
  { 0x0C, 0x03, 0x10, (char*)NM_BASE12, (char*)NM_USB,       "OHCI" },
  { 0x0C, 0x03, 0x20, (char*)NM_BASE12, "USB 2.0",           "EHCI" }, // PCI 2.3
  { 0x0C, 0x03, 0x80, (char*)NM_BASE12, (char*)NM_USB,       (char*)NM_GENERIC },
  { 0x0C, 0x03, 0xFE, (char*)NM_BASE12, (char*)NM_USB,       "Device" },
  { 0x0C, 0x04, 0x00, (char*)NM_BASE12, "Fibre Channel",     NM_N },
  { 0x0C, 0x05, 0x00, (char*)NM_BASE12, "SMBus (Sys Mgmt Bus)", NM_N },
  { 0x0C, 0x06, 0x00, (char*)NM_BASE12, "InfiniBand",        NM_N },            // PCI 2.3
  { 0x0C, 0x07, 0x00, (char*)NM_BASE12, (char*)NM_IPMI,      "SMIC" },          // PCI 2.3
  { 0x0C, 0x07, 0x01, (char*)NM_BASE12, (char*)NM_IPMI,      "Keyboard" },      // PCI 2.3
  { 0x0C, 0x07, 0x02, (char*)NM_BASE12, (char*)NM_IPMI,      "Block Transfer" },// PCI 2.3
  { 0x0C, 0x08, 0x00, (char*)NM_BASE12, "SERCOS",            "IEC 61491" },     // PCI 2.3
  { 0x0C, 0x09, 0x00, (char*)NM_BASE12, "CANBus",            NM_N },            // PCI 2.3
  { 0x0C, 0x80, 0x00, (char*)NM_BASE12, (char*)NM_OTHER,     NM_N }, // Not PCI

  // 0x0D - Wireless Controller
  { 0x0D, 0x00, 0x00, (char*)NM_BASE13, "iRDA",              NM_N },
  { 0x0D, 0x01, 0x00, (char*)NM_BASE13, "Consumer IR",       NM_N },
  { 0x0D, 0x10, 0x00, (char*)NM_BASE13, "RF",                NM_N },
  { 0x0D, 0x11, 0x00, (char*)NM_BASE13, "Bluetooth",         NM_N },      // PCI 2.3
  { 0x0D, 0x12, 0x00, (char*)NM_BASE13, "Broadband",         NM_N },      // PCI 2.3
  { 0x0D, 0x20, 0x00, (char*)NM_BASE13, (char*)NM_ETHERNET,  "802.11a" }, // PCI 3.0
  { 0x0D, 0x21, 0x00, (char*)NM_BASE13, (char*)NM_ETHERNET,  "802.11b" }, // PCI 3.0
  { 0x0D, 0x80, 0x00, (char*)NM_BASE13, (char*)NM_OTHER,     NM_N },

  // 0x0E - Intelligent IO Controller
  { 0x0E, 0x00, 0x00, (char*)NM_BASE14, NM_N,                NM_N },
  { 0x0E, 0x80, 0x00, (char*)NM_BASE14, (char*)NM_OTHER,     NM_N }, // Not PCI

  // 0x0F - Satellite Communication Controller
  { 0x0F, 0x01, 0x00, (char*)NM_BASE15, "TV",                NM_N },
  { 0x0F, 0x02, 0x00, (char*)NM_BASE15, "Audio",             NM_N },
  { 0x0F, 0x03, 0x00, (char*)NM_BASE15, "Voice",             NM_N },
  { 0x0F, 0x04, 0x00, (char*)NM_BASE15, "Data",              NM_N },
  { 0x0F, 0x80, 0x00, (char*)NM_BASE15, (char*)NM_OTHER,     NM_N }, // Not PCI

  // 0x10 - Encryption/Decryption Controller
  { 0x10, 0x00, 0x00, (char*)NM_BASE16, "Network & Computing", NM_N },
  { 0x10, 0x10, 0x00, (char*)NM_BASE16, "Entertainment",     NM_N },
  { 0x10, 0x80, 0x00, (char*)NM_BASE16, (char*)NM_OTHER,     NM_N },

  // 0x11 - Data Acquisiton/Signal Processing
  { 0x11, 0x00, 0x00, (char*)NM_BASE17, "DPIO modules",      NM_N },
  { 0x11, 0x01, 0x00, (char*)NM_BASE17, "Performance Counters", NM_N },   // PCI 2.3
  { 0x11, 0x10, 0x00, (char*)NM_BASE17, "Sync/Time/Freq Testing", NM_N }, // PCI 2.3
  { 0x11, 0x20, 0x00, (char*)NM_BASE17, "Management Card",   NM_N },      // PCI 2.3
  { 0x11, 0x80, 0x00, (char*)NM_BASE17, (char*)NM_OTHER,     NM_N },

  // 0xFF - Undefined (Not configured)
  { 0xFF, 0xFF, 0xFF, (char*)NM_UNKNOWN, NM_N,               NM_N },
};

// Use this value for loop control during searching:
#define	PCI_CLASSCODETABLE_LEN	(sizeof(PciClassCodeTable)/sizeof(PCI_CLASSCODETABLE))

//////////////////////////////
//
// use this to decode the PROGIF bits when
// baseclass = 0x01, "Mass Storage Controller" and
// subclass  = 0x01, "IDE"
//
// [0] corresponds to bit 0 and [7] corresponds to bit 7

static char* PciIdeBitTable[] =
{
  "Primary Operating Mode",
  "Primary Programming Indicator",
  "Secondary Operating Mode",
  "Secondary Programming Indicator",
  "Reserved 4"
  "Reserved 5",
  "Reserved 6",
  "Master IDE Device",
};

// Use this value for range checking
#define	PCI_IDEBITTABLE_LEN (sizeof(PciIdeBitTable)/sizeof(char*))
#define PciIdeBit(x)  PciIdeBitTable[((unsigned char)(x) & 0x7)]

static char *PciCommandFlags[] =
{
  "I/O Space",
  "Memory Space",
  "Bus Master",
  "Special Cycles",
  "Memory Write/Invalidate",
  "VGA Palette Snoop",
  "Parity Error Enable",
  "Wait Cycle Control",		// PCI 2.3
  "System Error Enable",
  "Fast Back-To-Back",
  "Interrupt Enable",			// PCI 2.3
  "Reserved 11",
  "Reserved 12",
  "Reserved 13",
  "Reserved 14",
  "Reserved 15"
};

// Use this value for loop control during searching:
#define	PCI_COMMANDFLAGS_LEN (sizeof(PciCommandFlags)/sizeof(char*))
#define PciCommandBit(x)  PciIdeBitTable[((unsigned char)(x) & 0x0F)]

static char *PciStatusFlags[] =
{
  "Reserved 0",
  "Reserved 1",
  "Reserved 2",
  "Interrupt Status",                 // PCI 2.3
  "Capability List",
  "66 MHz Capable",
  "User-Defined Feature",            	// Reserved in PCI 2.3
  "Fast Back-To-Back",
  "Master Data Parity Error",
  NM_N, // Devsel Timing (See Below)
  NM_N, // Devsel Timing (see below)
  "Signalled Target Abort",
  "Received Target Abort",
  "Received Master Abort",
  "Signalled System Error",
  "Detected Parity Error"
};

// Use this value for loop control during searching:
#define	PCI_STATUSFLAGS_LEN (sizeof(PciStatusFlags)/sizeof(char*))
#define PciStatusBit(x)  PciStatusFlags[((unsigned char)(x) & 0x0F)]

static char* PciDevSelFlags[] =
{
 "Fast Devsel Speed",     // TypeC
 "Medium Devsel Speed",   // TypeB
 "Slow Devsel Speed",     // TypeA
 "Reserved 9&10"
} ;

// Use this value for loop control during searching:
#define PCI_DEVSELFLAGS_LEN (sizeof(PciDevSelFlags)/sizeof(char*))
#define PciDevSelBit(x)  PciStatusFlags[((unsigned char)(x) & 0x03)]

// Capability ID list (PCI 2.2)
// 255 differnt types of optional capability types
//

static char *PciCapabilityIDs[] =
{
  "Reserved",
  "Power Management",
  "AGP",
  "VPD",
  "Slot Identification",
  "Message Based Interrupts",
  "CompactPCI Hot Swap",
  "PCI-X",
  "AMD",
  "Vendor Specific",
  "Debug Port",
  "CompactPCI CRCtl",
  "Hot Plug",
};

// Use this value for loop control during searching:
#define	PCI_EXTSTATUS_LEN (sizeof(PciExtStatusFlags)/sizeof(char*))

/*
///////////////////////////////////////////////////////////////////////////////
//
// PCI Text Class Code Table lookup code
//
// Args: baseid  - PCI base class code
//       subid   - PCI sub class code
//       progif  - PCI prog IF code
//
//       basestr - Returns base class description *pointer
//       substr  - Returns sub class description *pointer
//       progstr - Returns prog IF description *pointer
//
// Returns
//   nothing - all stings will contain pci descriptions or NM_N ("")
//
///////////////////////////////////////////////////////////////////////////////

void GetPCIID(char baseid,    char subid,    char progid,
              char **basestr, char **substr, char **progstr)
{
  int i;
  *basestr = *substr = *progstr = NM_N;

  for (i=0; i<sizeof(PciClassCodeTable)/sizeof(PCI_CLASSCODETABLE); ++i)
  {
    if (PciClassCodeTable[i].BaseClass == baseid)
    {
      if (!**basestr)
        *basestr = PciClassCodeTable[i].BaseDesc;
      if (PciClassCodeTable[i].SubClass == subid)
      {
        if (!**substr)
          *substr = PciClassCodeTable[i].SubDesc;
        if (PciClassCodeTable[i].ProgIf == progid)
        {
          *progstr = PciClassCodeTable[i].ProgDesc;
          return;
        }
      }
    }
  }
}

/////////////////////////////////////////////////////////////////////////
//
// PCI Text Class Code Table Lookup Code
//
// Args: classcode- PCI 32 bit class code
//       basedesc - Returns base class description
//       subdesc  - Returns sub class description
//       progdesc - Returns prog IF description
//
// Returns
//   nothing (all "desc" stings will contain pci descriptions or empty strings)
//
//////////////////////////////////////////////////////////////////////////

VOID GetPCIClass(long classcode,  char **base,  char **sub,  char **prog)
{
  unsigned char baseid, subid, progid;
  int  i;

  ///// Initialize Class Values
  *base = *sub = *prog = "";
  baseid =(long)  classcode >> 24;
  subid  =(long) (classcode & 0xFF0000L)>>16;
  progid =(long) (classcode & 0xFF00L)>>8;

  GetPCIID(unsigned char baseid,  unsgned char subid,  unsigned char progid
              char **basedesc,       char **subdesc,      char **progdesc)
}

/////////////////////////////////////////////////////////////////////////
//
// PCI Text Class Code Table Lookup Code
//
// Args: classcode- PCI 32 bit class code
//       basedesc - Returns base class description
//       subdesc  - Returns sub class description
//       progdesc - Returns prog IF description
//
// Returns
//   nothing (all "desc" stings will contain pci descriptions or empty strings)
//
//////////////////////////////////////////////////////////////////////////

VOID GetPCIClass(long classcode,  char **base,  char **sub,  char **prog)
{
  long baseid, subid, progid;
  int  i;

  ///// Initialize Class Values
  *base = *sub = *prog = "";
  baseid =(long)  classcode >> 24;
  subid  =(long) (classcode & 0xFF0000L)>>16;
  progid =(long) (classcode & 0xFF00L)>>8;

  for (i=0; i<sizeof(PciClassCodeTable)/sizeof(PCI_CLASSCODETABLE); ++i)
  {
    if (PciClassCodeTable[i].BaseClass == baseid)
    {
      if (!**base)
        *base = PciClassCodeTable[i].BaseDesc;
      if (PciClassCodeTable[i].SubClass == subid)
      {
        if (!**sub)
          *sub = PciClassCodeTable[i].SubDesc;
        if (PciClassCodeTable[i].ProgIf == progid)
        {
          *prog = PciClassCodeTable[i].ProgDesc;
          return;
        }
      }
    }
  }
}

*/


////////////////////

#endif // __PCICLASS_INCLUDE__




